@@ -16,11 +16,11 @@ import _primer3
 
 # ~~~~~~~ Check to insure that the environment is properly configured ~~~~~~~ #
 
-local_dir = os.path.dirname(os.path.realpath(__file__))
+LOCAL_DIR = os.path.dirname(os.path.realpath(__file__))
 
 if not os.environ.get('PRIMER3HOME'):
     try:
-        os.environ['PRIMER3HOME'] = pjoin(local_dir, 'src/primer3-2.3.6')
+        os.environ['PRIMER3HOME'] = pjoin(LOCAL_DIR, 'src/libprimer3')
     except:
         raise ImportError('PRIMER3HOME environmental variable is not set.')
 PRIMER3_HOME = os.environ.get('PRIMER3HOME')
@@ -28,7 +28,84 @@ PRIMER3_HOME = os.environ.get('PRIMER3HOME')
 
 # ~~~~~~~~~~~~~~~~ Load thermodynamic parameters into memory ~~~~~~~~~~~~~~~~ #
 
-_primer3.getThermoParams(pjoin(PRIMER3_HOME, 'src', 'primer3_config/'))
+_primer3.loadThermoParams(pjoin(PRIMER3_HOME, 'primer3_config/'))
+
+
+# ~~~~~~~~~~~~~ Lightweight low level bindings (return only Tm) ~~~~~~~~~~~~~ #
+
+
+def calcHairpinTm(seq, mv_conc=50.0, dv_conc=0.0, dntp_conc=0.8, dna_conc=50.0,
+                temp_c=37, max_loop=30):
+    ''' Calculate the hairpin formation thermodynamics of a DNA sequence.
+
+    Args:
+        seq (str)               : DNA sequence to analyze for hairpin formation
+
+    Kwargs:
+        mv_conc (float/int)     : Monovalent cation concentration (mM)
+        dv_conc (float/int)     : Divalent cation concentration (mM)
+        dntp_conc (float/int)   : dNTP concentration (mM)
+        dna_conc (float/int)    : DNA concentration (nM)
+        temp_c (int)            : Simulation temperature for dG (Celsius)
+        max_loop(int)           : Maximum size of loops in the structure
+
+    Returns:
+        The Tm of the structure in degress C.
+
+    '''
+
+    return _primer3.calcThermoTm(seq, seq, 4, mv_conc, dv_conc, dntp_conc,
+                                 dna_conc, temp_c + 273.15, max_loop, 0, 0)
+
+
+def calcHomodimerTm(seq, mv_conc=50, dv_conc=0, dntp_conc=0.8, dna_conc=50,
+                  temp_c=37, max_loop=30):
+    ''' Calculate the homodimerization thermodynamics of a DNA sequence.
+
+    Args:
+        seq (str)               : DNA sequence to analyze for homodimer
+                                  formation calculations
+
+    Kwargs:
+        mv_conc (float/int)     : Monovalent cation concentration (mM)
+        dv_conc (float/int)     : Divalent cation concentration (mM)
+        dntp_conc (float/int)   : dNTP concentration (mM)
+        dna_conc (float/int)    : DNA concentration (nM)
+        temp_c (int)            : Simulation temperature for dG (Celsius)
+        max_loop (int)          : Maximum size of loops in the structure
+
+    Returns:
+        The Tm of the structure in degress C.
+
+    '''
+    return _primer3.calcThermoTm(seq, seq, 1, mv_conc, dv_conc, dntp_conc,
+                                 dna_conc, temp_c + 273.15, max_loop, 0, 0)
+
+
+def calcHeterodimerTm(seq1, seq2, mv_conc=50, dv_conc=0, dntp_conc=0.8,
+                      dna_conc=50, temp_c=37, max_loop=30):
+    ''' Calculate the heterodimerization thermodynamics of two DNA sequences.
+
+    Args:
+        seq1 (str)              : First DNA sequence to analyze for heterodimer
+                                  formation
+        seq2 (str)              : Second DNA sequence to analyze for
+                                  heterodimer formation
+
+    Kwargs:
+        mv_conc (float/int)     : Monovalent cation concentration (mM)
+        dv_conc (float/int)     : Divalent cation concentration (mM)
+        dntp_conc (float/int)   : dNTP concentration (mM)
+        dna_conc (float/int)    : DNA concentration (nM)
+        temp_c (int)            : Simulation temperature for dG (Celsius)
+        max_loop(int)           : Maximum size of loops in the structure
+
+    Returns:
+        The Tm of the structure in degress C.
+
+    '''
+    return _primer3.calcThermoTm(seq1, seq2, 1, mv_conc, dv_conc, dntp_conc,
+                                 dna_conc, temp_c + 273.15, max_loop, 0, 0)
 
 
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Low level bindings ~~~~~~~~~~~~~~~~~~~~~~~~~~~ #
@@ -36,6 +113,7 @@ _primer3.getThermoParams(pjoin(PRIMER3_HOME, 'src', 'primer3_config/'))
 # Named tuple returned by all low-level bindings (if a structure is not
 # present, the functions simply return `None`)
 THERMORESULT = namedtuple('thermoresult', [
+    'structure_found',  # True if a structure was found
     'tm',               # Melting temperature (deg. Celsius)
     'ds',               # Entropy (cal/(K*mol))
     'dh',               # Enthalpy (kcal/mol)
@@ -44,6 +122,7 @@ THERMORESULT = namedtuple('thermoresult', [
     'align_end_2']      # Last alignment position in the 2nd sequence.
 )
 
+NULLTHERMORESULT = THERMORESULT(False, 0, 0, 0, 0, 0, 0)
 
 def _calcThermo(seq1, seq2, calc_type=0, mv_conc=50, dv_conc=0, dntp_conc=0.8,
                    dna_conc=50, temp_c=37, max_loop=30):
@@ -51,11 +130,11 @@ def _calcThermo(seq1, seq2, calc_type=0, mv_conc=50, dv_conc=0, dntp_conc=0.8,
                                 dntp_conc, dna_conc, temp_c + 273.15,
                                 max_loop, 0, 0)
     if res[1] == 1: # No structure found
-        if res[0] != '':
+        if len(res[0]) > 0:
             raise IOError('Primer3 error: {}'.format(res[0]))
         else:
-            return None
-    return THERMORESULT(*res[2:])
+            return NULLTHERMORESULT
+    return THERMORESULT(True, *res[2:])
 
 
 def calcHairpin(seq, mv_conc=50.0, dv_conc=0.0, dntp_conc=0.8, dna_conc=50.0,
